/*===================================================================*/
/*                                                                   */
/*  InfoNES_System.h : The function which depends on a system        */
/*                                                                   */
/*  2000/05/29  InfoNES Project ( based on pNesX )                   */
/*                                                                   */
/*===================================================================*/

/*-------------------------------------------------------------------*/
/*  Include files                                                    */
/*-------------------------------------------------------------------*/

#define DWORD DWORD_
#include <stdlib.h>
#include <stdio.h>
#include <fcntl.h>
#include <string.h>
#include <rpcss.h>

#include "InfoNES.h"
#include "InfoNES_System.h"
#include "InfoNES_pAPU.h"

/*-------------------------------------------------------------------*/
/*  Palette data                                                     */
/*-------------------------------------------------------------------*/
//extern WORD NesPalette[];
WORD NesPalette[ 64 ] =
{
  0x39ce, 0x1071, 0x0015, 0x2013, 0x440e, 0x5402, 0x5000, 0x3c20,
  0x20a0, 0x0100, 0x0140, 0x00e2, 0x0ceb, 0x0000, 0x0000, 0x0000,
  0x5ef7, 0x01dd, 0x10fd, 0x401e, 0x5c17, 0x700b, 0x6ca0, 0x6521,
  0x45c0, 0x0240, 0x02a0, 0x0247, 0x0211, 0x0000, 0x0000, 0x0000,
  0x7fff, 0x1eff, 0x2e5f, 0x223f, 0x79ff, 0x7dd6, 0x7dcc, 0x7e67,
  0x7ae7, 0x4342, 0x2769, 0x2ff3, 0x03bb, 0x0000, 0x0000, 0x0000,
  0x7fff, 0x579f, 0x635f, 0x6b3f, 0x7f1f, 0x7f1b, 0x7ef6, 0x7f75,
  0x7f94, 0x73f4, 0x57d7, 0x5bf9, 0x4ffe, 0x0000, 0x0000, 0x0000
};

/*-------------------------------------------------------------------*/
/*  Grobal variable                                                  */
/*-------------------------------------------------------------------*/

static int offx, offy;

/*-------------------------------------------------------------------*/
/*  Function prototypes                                              */
/*-------------------------------------------------------------------*/

#if 0
static int printf(char *format, ...)
/* output cosole */
{
	int i, I;
	va_list ap;

	va_start(ap, format);
	I = vsprintf(strbuf, format, ap);
	va_end(ap);
	for(i = 0; i < I; i++){
		putch(strbuf[i]);
	}
	return I;
}
#endif

/*===================================================================*/
/*                                                                   */
/*                main() : Application main                          */
/*                                                                   */
/*===================================================================*/

int main( int argc, char** argv )
{
	GSSTATUS gs;
	int file;

	/* Command line */
	if (argc < 2) {
		puts("InfoNES for Pekoe");
		puts("copyright(c)2004,bayside.");
		puts("");
		puts("usage: infones.com [*.nes]");
		puts("keyconfigration:");
		puts("    START  : 's'");
		puts("    SELECT : 'a'");
		puts("    buttonA: 'x'");
		puts("    buttonB: 'z'");
		puts("    SLOWER : 'k'");
		puts("    FASTER : 'l'");
		puts("    EXIT   : 'ESC'");
		return 0;
	}

	/* File exists check */
	if ((file = open(argv[1], O_RDONLY, 0)) < 0) {
		return 0;
	} else {
		close(file);
	}

	/* Open ROM file */
	InfoNES_Load(argv[1]);

	/* Set screen mode */
	sys_gs_get_status(&gs);
	if (gs.bpp <= 8) {
		sys_gs_set_mode(gs.width, gs.height, 8);
	} else {
		sys_gs_set_mode(gs.width, gs.height, gs.bpp);
	}
	offx = (gs.width  - NES_DISP_WIDTH ) / 2;
	offy = (gs.height - NES_DISP_HEIGHT) / 2;

	/* Set frame skip */
	FrameSkip = 5;

	/* The main loop of InfoNES */ 
	InfoNES_Main();

	return 0;
}

/* Menu screen */
int InfoNES_Menu(){
	return 0;
}

/* Read ROM image file */
int InfoNES_ReadRom( const char *pszFileName ){
	int i, file1, filesize1 = 0, readsize1 = 0;
	BYTE *fp1;
	
	if ((file1 = open(pszFileName, O_RDONLY, 0)) < 0) return 1;
	filesize1 = lseek(file1, 0, 2);
	lseek(file1, 0, 0); // goto top
	fp1 = (BYTE *)malloc(filesize1);
	if ((readsize1 = read(file1, (char*)fp1, filesize1)) <= 0) return 1;

	/* Read header */
	for (i = 0; i < 4; i++) NesHeader.byID[i] = *fp1++;
	NesHeader.byRomSize  = *fp1++;
	NesHeader.byVRomSize = *fp1++;
	NesHeader.byInfo1    = *fp1++;
	NesHeader.byInfo2    = *fp1++;
	for (i = 0; i < 8; i++) NesHeader.byReserve[i] = *fp1++;

	/* Clear SRAM */
	for (i = 0; i < SRAM_SIZE; i++) SRAM[i] = 0;

	/* If trainer presents Read Triner at 0x7000-0x71ff */
	if ( NesHeader.byInfo1 & 4 ) {
		for (i = 0; i < 512; i++)
			//SRAM[0x1000 + i] = *fp1++;
			*fp1++;
	}

	/* Allocate Memory for ROM Image */
	ROM = (BYTE *)malloc( NesHeader.byRomSize * 0x4000 );

	/* Read ROM Image */
	for(i = 0; i < NesHeader.byRomSize * 0x4000; i++)
		ROM[i] = *fp1++;

	if ( NesHeader.byVRomSize > 0 ){
		/* Allocate Memory for VROM Image */
		VROM = (BYTE *)malloc( NesHeader.byVRomSize * 0x2000 );

		/* Read VROM Image */
		for(i = 0; i < NesHeader.byVRomSize * 0x2000; i++)
			VROM[i] = *fp1++;
	}

	/* Successful */
	return 0;
}

/* Release a memory for ROM */
void InfoNES_ReleaseRom(){
/*
 *  Release a memory for ROM
 *
 */

	if ( ROM ) {
		free( ROM );
		ROM = NULL;
	}

	if ( VROM ) {
		free( VROM );
		VROM = NULL;
	}
}

/* Transfer the contents of work frame on the screen */
void InfoNES_LoadFrame(){
	WORD rgb;
	int x, y;
	
	/* Copy WorkFrame to VRAM */
	for(y = 0; y < NES_DISP_HEIGHT; y++){
		for(x = 0; x < NES_DISP_WIDTH; x++){
			rgb = WorkFrame[y * NES_DISP_WIDTH + x];
			//bitmap[(y * NES_DISP_WIDTH + x) * 3    ] = (rgb & 0x1F) << 3;
			//bitmap[(y * NES_DISP_WIDTH + x) * 3 + 1] = (rgb & 0x3E0) >> 2;
			//bitmap[(y * NES_DISP_WIDTH + x) * 3 + 2] = (rgb & 0x7C00) >> 7;
			sys_gs_set_pixel_RGB(x + offx, y + offy, 
				((rgb & 0x7C00) >> 7) << 16 | ((rgb & 0x3E0) >> 2) << 8 | ((rgb & 0x1F) << 3));
		}
	}
}

/* Get a joypad state */
void InfoNES_PadState( DWORD *pdwPad1, DWORD *pdwPad2, DWORD *pdwSystem ){
	KEY_PACKET kpacket;

	//*pdwPad1 = 0;
	//*pdwPad2 = 0;
	
	if (sys_fs_read(devid_rpcss, port_keybd, 0, &kpacket, 1) > 0) {
		//printf("key: %d %d\n", kpacket.keycode, kpacket.shift_state);
		if (kpacket.keycode == 0) { // ESC
			exit(0);
		}
		// key typed
		//if (kpacket.keycode <= 0x4B) {
			if (kpacket.keycode == 0x43) { // UP
				*pdwPad1 |= 0x10;
			} else if (kpacket.keycode == 0x4B) { // DOWN
				*pdwPad1 |= 0x20;
			} else if (kpacket.keycode == 0x46) { // LEFT
				*pdwPad1 |= 0x40;
			} else if (kpacket.keycode == 0x48) { // RIGHT
				*pdwPad1 |= 0x80;
			} else if (kpacket.keycode == 0x29) { // B (a)
				*pdwPad1 |= 0x2;
			} else if (kpacket.keycode == 0x2A) { // A (s)
				*pdwPad1 |= 0x1;
			} else if (kpacket.keycode == 0x1D) { // SELECT (z)
				*pdwPad1 |= 0x4;
			} else if (kpacket.keycode == 0x1E) { // START (x)
				*pdwPad1 |= 0x8;
			} else if (kpacket.keycode == 0x24) { // SPEED DOWN (k)
				if (FrameSkip > 0) {
					FrameSkip--;
				}
			} else if (kpacket.keycode == 0x25) { // SPEED UP (l)
				FrameSkip++;
			}
		// key released
		//} else if (kpacket.keycode >= 0x80000000) {
			else if (kpacket.keycode == 0x80000043) { // UP
				*pdwPad1 ^= 0x10;
			} else if (kpacket.keycode == 0x8000004B) { // DOWN
				*pdwPad1 ^= 0x20;
			} else if (kpacket.keycode == 0x80000046) { // LEFT
				*pdwPad1 ^= 0x40;
			} else if (kpacket.keycode == 0x80000048) { // RIGHT
				*pdwPad1 ^= 0x80;
			} else if (kpacket.keycode == 0x80000029) { // B (a)
				*pdwPad1 ^= 0x2;
			} else if (kpacket.keycode == 0x8000002A) { // A (s)
				*pdwPad1 ^= 0x1;
			} else if (kpacket.keycode == 0x8000001D) { // SELECT (z)
				*pdwPad1 ^= 0x4;
			} else if (kpacket.keycode == 0x8000001E) { // START (x)
				*pdwPad1 ^= 0x8;
			}
		//}
	}
}

/* memcpy */
void *InfoNES_MemoryCopy( void *dest, const void *src, int count ){
	return memcpy(dest, src, count);
}

/* memset */
void *InfoNES_MemorySet( void *dest, int c, int count ){
	return memset(dest, c, count);
}

/* Print debug message */
void InfoNES_DebugPrint( char *pszMsg ){
	printf( "InfoNES:%s\n", pszMsg );
}

/* Wait */
void InfoNES_Wait(){
}

/* Sound Initialize */
void InfoNES_SoundInit( void ){
}

/* Sound Open */
int InfoNES_SoundOpen( int samples_per_sync, int sample_rate ){
	return 0;
}

/* Sound Close */
void InfoNES_SoundClose( void ){
}

/* Sound Output 5 Waves - 2 Pulse, 1 Triangle, 1 Noise, 1 DPCM */
void InfoNES_SoundOutput(int samples, BYTE *wave1, BYTE *wave2, BYTE *wave3, BYTE *wave4, BYTE *wave5){
}

/* Print system message */
void InfoNES_MessageBox( char *pszMsg, ... ){
	printf( "InfoNES:%s\n", pszMsg );
}
